#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphSummary
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.paragraph.header import HeaderParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu
from pyams_content.component.paragraph.zmi.container import ParagraphContainerView
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='add-header-paragraph.menu', context=IParagraphContainerTarget, view=ParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=40)
class HeaderParagraphAddMenu(BaseParagraphAddMenu):
    """Header paragraph add menu"""

    label = _("Add header paragraph...")
    label_css_class = 'fa fa-fw fa-header'
    url = 'add-header-paragraph.html'
    paragraph_type = 'Header'


@pagelet_config(name='add-header-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HeaderParagraphAddForm(AdminDialogAddForm):
    """Header paragraph add form"""

    legend = _("Add new header paragraph")
    icon_css_class = 'fa fa-fw fa-header'

    fields = field.Fields(IHeaderParagraph).select('header')
    ajax_handler = 'add-header-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HeaderParagraphAddForm, self).updateWidgets(prefix)
        if 'header' in self.widgets:
            self.widgets['header'].widget_css_class = 'textarea height-100'

    def create(self, data):
        return HeaderParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-header-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HeaderParagraphAJAXAddForm(BaseParagraphAJAXAddForm, HeaderParagraphAddForm):
    """Header paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IHeaderParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HeaderParagraphPropertiesEditForm(AdminDialogEditForm):
    """Header paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit header paragraph properties")
    icon_css_class = 'fa fa-fw fa-header'

    fields = field.Fields(IHeaderParagraph).select('header')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HeaderParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'header' in self.widgets:
            self.widgets['header'].widget_css_class = 'textarea height-100'


@view_config(name='properties.json', context=IHeaderParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HeaderParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, HeaderParagraphPropertiesEditForm):
    """Header paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HeaderParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'header' in changes.get(IHeaderParagraph, ()):
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                            'object_name': self.context.__name__,
                            'title': II18n(self.context).query_attribute('title', request=self.request),
                            'visible': self.context.visible}
            })
        return output


@adapter_config(context=(IHeaderParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class HeaderParagraphInnerEditForm(HeaderParagraphPropertiesEditForm):
    """Header paragraph inner edit form"""

    legend = None
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


#
# HTML paragraph summary
#

@adapter_config(context=(IHeaderParagraph, IPyAMSLayer), provides=IParagraphSummary)
@template_config(template='templates/header-summary.pt', layer=IPyAMSLayer)
class HeaderParagraphSummary(BaseContentProvider):
    """Header paragraph renderer"""

    language = None

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('header', ):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('header', ):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))
